function [pl]=funcPML(data)
% Computation of the solution of the free Schroedinger equation
% by Crank-Nicolson scheme with finite differences method
% with Absorbing Boundary conditions : PML

% data is a cell
%  |
%  |--- data.x0
%  |--- data.x1
%  |--- data.N
%  |--- data.tfinal
%  |--- data.dt
%  |--- data.k0  
%  |--- data.m
%  |--- data.axe1  : where to plot |u|
%  |--- data.axe2  : where to plot contour of log10|u|
%  |--- data.txtiter    : handle on textiter
%  |--- data.txtnbiter  : handle on textnbiter

%  clear all;

% Data
% Computational domain : [x0,x1]
x0=data.x0;
x1=data.x1;
N=data.N;
geo.x = linspace(x0,x1,N);
x = geo.x.';
dx = x(2)-x(1);

%PML parameters
M_Left = ceil(0.2*N); 
M_Right = ceil(0.2*N);
sigma = 1;

PML.Left.M = M_Left;
PML.Left.sigma = sigma;
PML.Right.M = M_Right;
PML.Right.sigma = sigma;
xPML = [ [-40:1:-1]*dx+x0 x.' x1+[1:40]*dx]; 
% Final time of computation
% dt : time step
% niter : number of iterations
tfinal=data.tfinal;
dt=data.dt;
niter=tfinal/dt;
set(data.txtnbiter,'String',num2str(niter));  

% Wave number
k0=data.k0;

%--------------------------------------------------------------
%--------------------------------------------------------------
SysMat = getSysMat(geo,PML);

KK = i/dt*SysMat.M + 1/2*SysMat.A;
MM = i/dt*SysMat.M - 1/2*SysMat.A;

[L, U, P, Q] = lu(KK);
 
% Initial datum
%u_n=initialDatum(x,k0);
PROJECTOR = sparse([1:N]+PML.Left.M,[1:N],ones(1,N),N+PML.Right.M+PML.Left.M,N);
u_n_D = initialDatum(x,k0);
u_n = PROJECTOR * u_n_D;

sav=1;
% pl : used to store |u^n|
pl=zeros(niter/25,N);
pl2=pl;
pl(1,:)=abs(u_n_D)';
tl=zeros(niter/25,1);
tl(1)=0;


%--------------------------------------------------------------
%--------------------------------------------------------------

% Iterations
for k=1:niter,
  
  u_n1 = Q*(U\(L\(P*(MM*u_n))));
  u_n = u_n1;
  
% Display
  if rem(k,25)==0
% $$$       fprintf('Iteration %d\n',k);
% $$$       plot(x,abs(u1));axis([x0 x1 0 1]);drawnow;
    sav=sav+1;
    pl(sav,:)=(abs(PROJECTOR.'*u_n))';
    tl(sav)=k*dt;
    plot(x,abs(PROJECTOR.'*u_n),'parent',data.axe1);axis(data.axe1,[x0 x1 0 1]);drawnow;
    set(data.txtiter,'String',num2str(k));
    if rem(sav,20)==0
      pl2(1:sav,:)=log10(pl(1:sav,:));
      contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
	       'parent',data.axe2);
      axis(data.axe2,[x0-1 x1+1 0 tfinal]);
      h=colorbar('peer',data.axe2);
      set(h,'fontsize',7);
    end
  end
end
% $$$   figure(2);
% $$$   mesh(x,tl,pl);view(160,60);
pl2(1:sav,:)=log10(pl(1:sav,:));
contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
	 'parent',data.axe2);
axis(data.axe2,[x0-1 x1+1 0 tfinal]);
h=colorbar('peer',data.axe2);
set(h,'fontsize',7);

function [SysMat] = getSysMat(geo,PML)

dx = geo.x(2)-geo.x(1);

np = size(geo.x,2);

npml_l = PML.Left.M;
npml_r = PML.Right.M;

ndof = np+npml_l+npml_r;

gammapL = (1+i*PML.Left.sigma);
gammapR = (1+i*PML.Right.sigma);

e_pml_l = [gammapL*ones(npml_l+1,1); zeros(ndof-npml_l-1,1)];
e_pml_r = [zeros(ndof-npml_r-1,1); gammapR*ones(npml_r+1,1)];
e_comp = [zeros(npml_l,1); ones(np,1); zeros(npml_r,1)];
em = e_comp+e_pml_r+e_pml_l;
M = spdiags(em,0,ndof,ndof);
  
SysMat.M = M; 
comp = 1;
pml_l = 1;
pml_r = 1;

el_pml_l = pml_l * [(1/(1+i*PML.Left.sigma))*ones(npml_l-1,1); ...
		  (2/(1+i*PML.Left.sigma)); zeros(ndof-npml_l,1)];
el_comp  =  comp * [zeros(npml_l,1); ones(np-2,1); 2; zeros(npml_r+1,1)];
el_pml_r = pml_r * [zeros(ndof-npml_r-1,1); (1/(1+i*PML.Right.sigma))*ones(npml_r+1,1)];
el = el_pml_l + el_comp + el_pml_r;

em_pml_l = pml_l * [(-2/(1+i*PML.Left.sigma))*ones(npml_l+1,1);zeros(ndof-npml_l-1,1)];
em_comp  =  comp*[zeros(npml_l,1); -2*ones(np,1); zeros(npml_r,1)];
em_pml_r = pml_r * [zeros(ndof-npml_r-1,1); (-2/(1+i*PML.Right.sigma))*ones(npml_r+1,1)];
em = em_pml_l + em_comp + em_pml_r;

eu_pml_l = pml_l * [(1/(1+i*PML.Left.sigma))*ones(npml_l+1,1);zeros(ndof-npml_l-1,1)];
eu_comp  =  comp * [zeros(npml_l+1,1);2; ones(np-2,1); zeros(npml_r,1)];
eu_pml_r = pml_r * [zeros(ndof-npml_r,1);(2/(1+i*PML.Right.sigma)) ;(1/(1+i*PML.Right.sigma))*ones(npml_r-1,1)];
eu = eu_pml_l + eu_comp + eu_pml_r;

A = spdiags([el./dx^2, em./dx^2, eu./dx^2],[-1:1], np+npml_l+npml_r,np+npml_l+npml_r);

SysMat.A = A;