function [out]=funcarnold(data)
% Antoine-Besse April 2006
% Computation of the solution of the free Schroedinger equation
% by Arnold-Ehrhart scheme with Absorbing Boundary conditions

% i a d/dt u + b d/dx^2 u = Vu

% data is a cell
%  |
%  |--- data.a           parameter of the eqaution                   real scalar
%  |--- data.b           parameter of the equation                   real scalar
%  |--- data.sigma       parameter of the initial datum              real scalar    
%  |--- data.xc          parameter of the initial datum              real scalar
%  |--- data.x0          left boundary of the computational domain   real scalar
%  |--- data.x1          right boundary of the computational domain  real scalar
%  |--- data.N           number of interior grid points integer
%  |--- data.tfinal      final time                                  real scalar
%  |--- data.dt          time step size                              real scalar
%  |--- data.k0          parameter of the initial datum              real scalar 
%  |--- data.m           not in use
%  |--- data.initDatum   initialdatum                                matlab string
%  |--- data.axe1       : where to plot |u|
%  |--- data.axe2       : where to plot contour of log10|u|
%  |--- data.txtiter    : handle on textiter
%  |--- data.txtnbiter  : handle on textnbiter

%  clear all;
% Setting default values
if ~isfield(data,'PlotFlag')
  data.PlotFlag = logical(1);
end
if ~isfield(data,'ReturnFlag')
  data.ReturnFlag = 1;
end

% Data
% Computational domain : [x0,x1]
% N+2 points : extremity set on points 1 and N+2
a=data.a;
b=data.b;
savenb=data.savenb;
sigma=data.sigma;
xc=data.xc;
x0=data.x0; 
x1=data.x1;
N=data.N;
dx=(x1-x0)/(N+1);
% Final time of computation
% dt : time step
% niter : number of iterations
tfinal=data.tfinal;
dt=data.dt;
niter=ceil(tfinal/dt);
if data.PlotFlag
  set(data.txtnbiter,'String',num2str(niter));  
end
% Wave number
k0=data.k0;

x=(x0:dx:x1)';
gridx = x;
% Initial datum
%u1=initialDatum(x,k0);
u=eval(data.initDatum);  

% try
%   V = potential_func(data.xc);
%   a = data.a;
%   b = data.b;
%   xc = data.xc;
%   k0 = data.k0;
%   sigma = data.sigma;
%   t = 0;
%   u = eval(data.exactCpot); 
% catch
%    warning('Using initDatum') 
% end
%--------------------------------------------------------------
%--------------------------------------------------------------
%
% Computation of convolution terms
%
R=2*a*dx*dx/dt/b;

Vl=potential_func(x0); % Potential value on xl
sl=zeros(niter+2,1);
sigmal=dx*dx*Vl/b;
argum=2*R*(sigmal+2)/(R^2-4*sigmal-sigmal^2);
%   if (argum<0)
%     phil=atan(argum)+pi;
%   else
%     phil=atan(argum);
%   end
Zl=(R^2-4*sigmal-sigmal^2)+i*(2*R*(sigmal+2));
phil=angle(Zl);
mul=(R^2+4*sigmal+sigmal^2)/sqrt((R^2+sigmal^2)*(R^2+(sigmal+4)^2));
alphal=(i/2)*((R^2+sigmal^2)*(R^2+(sigmal+4)^2))^0.25*exp(i*phil/2);
l1l=legendre(0,mul);
sl(1)=1-i*R/2+sigmal/2-alphal*l1l(1);
l1l=legendre(1,mul);
sl(2)=1+i*R/2+sigmal/2+alphal*exp(-i*phil)*l1l(1);
l1l=legendre(2,mul);
l2l=legendre(0,mul);
sl(3)=alphal*exp(-i*2*phil)*(l1l(1)-l2l(1))/3;
lambdal=exp(-i*phil);

Vr=potential_func(x1); % Potential value on xr  
sr=zeros(niter+2,1);
sigmar=dx*dx*Vr/b;
argum=2*R*(sigmar+2)/(R^2-4*sigmar-sigmar^2);
%   if (argum<0)
%     phir=atan(argum)+pi;
%   else
%     phir=atan(argum);
%   end
Zr=(R^2-4*sigmar-sigmar^2)+i*(2*R*(sigmar+2));
phir=angle(Zr);
mur=(R^2+4*sigmar+sigmar^2)/sqrt((R^2+sigmar^2)*(R^2+(sigmar+4)^2));
alphar=(i/2)*((R^2+sigmar^2)*(R^2+(sigmar+4)^2))^0.25*exp(i*phir/2);
l1r=legendre(0,mur);
sr(1)=1-i*R/2+sigmar/2-alphar*l1r(1);
l1r=legendre(1,mur);
sr(2)=1+i*R/2+sigmar/2+alphar*exp(-i*phir)*l1r(1);
l1r=legendre(2,mur);
l2r=legendre(0,mur);
sr(3)=alphar*exp(-i*2*phir)*(l1r(1)-l2r(1))/3;
lambdar=exp(-i*phir);

for j=3:niter+1
%    l1=legendre(j,mu);l2=legendre(j-2,mu);
%    s(j+1)=alpha*exp(-i*j*phi)*(l1(1)-l2(1))/(2*j-1);
  sl(j+1)=(2*j-3)*mul*lambdal*sl(j)/j-(j-3)*lambdal^2*sl(j-1)/j;
  sr(j+1)=(2*j-3)*mur*lambdar*sr(j)/j-(j-3)*lambdar^2*sr(j-1)/j;
end

sl = sl./dx;
sr = sr./dx;
%-----------------------------------------------------------
% Matrices assembly
%-----------------------------------------------------------
pot=potential_func(x);
e=ones(N+2,1);
e1=(b*0.5/dx)*e;
e2=(a*i*dx/dt-b/dx)*e;
e3=(a*i*dx/dt+b/dx)*e;
A=spdiags([e1 e2-dx*pot/2 e1],[-1 0 1],N+2,N+2);

A(1,1)=-sl(1);
A(1,2)=1/dx;
A(N+2,N+2)=-sr(1);
A(N+2,N+1)=A(1,2);
%
B=spdiags([-e1 e3+dx*pot/2 -e1],[-1 0 1],N+2,N+2);
B(1,:)=0;
B(N+2,:)=0;

%V0=[ ];
%V1=[ ];
%W0=[ ];
%W1=[ ];
V0 = zeros(1,niter);
V1 = zeros(1,niter);

SM=zeros(N+2,1);
SM(1)=0;
SM(N+2)=0;

% LU factorization of matrix A

%[L,U]=lu(A);
[LA,UA,pA,qA] = lu(A,'vector');

sav=1;
% pl : used to store |u^n|
pl=zeros(niter/savenb,N+2); 
pl(1,:)=u.';
tl=zeros(niter/savenb,1);
tl(1)=0;
% ERR : relative error in L2 norm
K=[0];ERR=[0];tmp=0;
% l2 norm computed by Simpson quadrature rule
l2=l2simpson(dx,abs(u));
% Vector to store the evolution of the L2 norm of the approximate
% solution
L2NORM=[l2];
% Vector to store the evolution of the L2 norm of the exact solution
L2NORMEX=[l2];

%--------------------------------------------------------------
%--------------------------------------------------------------

MaxV=max(potential_func(x));
if (MaxV==0)
  MaxV=1;
end;
% Iterations
for k=1:niter,
% u1 means u^n
% u2 means u^{n+1}
% Resolution of the linear system

%   y=L\(B*u1+SM);
%   u2=U\y;
  rhs = B*u+SM;
  u(qA,:)= UA\(LA\rhs(pA,:));
% Storage of the boundary terms
%  V0=[V0 u(1)];  W0=[W0 u(2)/dx];
%  V1=[V1 u(N+2)];W1=[W1 u(N+1)/dx];
  V0(k) = u(1);  
  V1(k) = u(N+2);
% Replacement of the old u^n by u^(n+1)
% Computation of the futur R.H.S by convolution
  saml=sl(2:k+1);
  samr=sr(2:k+1);
%   SM(1)=fliplr(V0(1:k))*saml-W0(k);
%   SM(N+2)=fliplr(V1(1:k))*samr-W1(k);
  SM(1)   = fliplr(V0(1:k))*saml-u(2)/dx;
  SM(N+2) = fliplr(V1(1:k))*samr-u(N+1)/dx;
  
% Display
  if rem(k,savenb)==0
    sav=sav+1;
    pl(sav,:)=u.';
    tl(sav)=k*dt;
    if   data.PlotFlag
      plot(x,abs(u),x,potential_func(x)/MaxV,'r:','parent',data.axe1);axis(data.axe1,[x0 x1 0 1.1]);drawnow;
      set(data.txtiter,'String',num2str(k));
      if rem(sav,20)==0
	pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
	contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
		 'parent',data.axe2);
	axis(data.axe2,[x0 x1 0 tfinal]);
	h=colorbar('peer',data.axe2);
	set(h,'fontsize',7);
      end
% $$$       tmp=cputime;
    end
  end
end

if data.PlotFlag
  pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
  contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
	   'parent',data.axe2);
  axis(data.axe2,[x0 x1 0 tfinal]);
  h=colorbar('peer',data.axe2);
  set(h,'fontsize',7);
end
% Output
if data.ReturnFlag == 1
  out = pl;
elseif data.ReturnFlag == 2
  out.pl = pl;
  out.tl = tl;
  out.gridx = gridx;
else
  error(sprintf('Unknown ReturnFlag %g',ReturnFlag))
end