function [out]=funcarnold2(data)
% Antoine-Besse April 2006
% Computation of the solution of the free Schroedinger equation
% by Arnold-Ehrhart scheme with Absorbing Boundary conditions

% data is a cell
%  |
%  |--- data.x0
%  |--- data.x1
%  |--- data.N
%  |--- data.tfinal
%  |--- data.dt
%  |--- data.k0  
%  |--- data.m
%  |--- data.axe1  : where to plot |u|
%  |--- data.axe2  : where to plot contour of log10|u|
%  |--- data.txtiter    : handle on textiter
%  |--- data.txtnbiter  : handle on textnbiter

%  clear all;

% Setting default values
if ~isfield(data,'PlotFlag')
  data.PlotFlag = logical(1);
end
if ~isfield(data,'ReturnFlag')
  data.ReturnFlag = 1;
end

% Data
% Computational domain : [x0,x1]
% N+2 points : extremity set on points 1 and N+2
x0=data.x0; 
x1=data.x1;
N=data.N;
dx=(x1-x0)/(N+1);
gridx = [x0:dx:x1]; 
% Final time of computation
% dt : time step
% niter : number of iterations
tfinal=data.tfinal;
dt=data.dt;
niter=tfinal/dt;
if data.PlotFlag
  set(data.txtnbiter,'String',num2str(niter));  
end
% Wave number
k0=data.k0;

x=(x0:dx:x1)';

% Initial datum
u1=initialDatum(x,k0);
u2=u1;

%--------------------------------------------------------------
%--------------------------------------------------------------

%
% Computation of convolution terms
%
s=zeros(niter+2,1);
R=2*dx*dx/dt;
phi=atan(4/R);mu=R/sqrt(R^2+16);
alpha=(i/2)*(R^2*(R^2+16))^0.25*exp(i*phi/2);
l1=legendre(0,mu);
s(1)=1-i*R/2-alpha*l1(1);
l1=legendre(1,mu);
s(2)=1+i*R/2+alpha*exp(-i*phi)*l1(1);
l1=legendre(2,mu);l2=legendre(0,mu);
s(3)=alpha*exp(-i*2*phi)*(l1(1)-l2(1))/3;
lambda=exp(-i*phi);
for j=3:niter+1
%    l1=legendre(j,mu);l2=legendre(j-2,mu);
%    s(j+1)=alpha*exp(-i*j*phi)*(l1(1)-l2(1))/(2*j-1);
  s(j+1)=(2*j-3)*mu*lambda*s(j)/j-(j-3)*lambda^2*s(j-1)/j;
end

%-----------------------------------------------------------
% Matrices assembly
%-----------------------------------------------------------
e=ones(N+2,1);
e1=(0.5/dx^2)*e;
e2=(i/dt-1/dx^2)*e;
e3=(i/dt+1/dx^2)*e;
A=spdiags([e1 e2 e1],[-1 0 1],N+2,N+2);
A(1,1)=-s(1);
A(1,2)=1;
A(N+2,N+2)=A(1,1);
A(N+2,N+1)=A(1,2);
%
B=spdiags([-e1 e3 -e1],[-1 0 1],N+2,N+2);
B(1,:)=0;
B(N+2,:)=0;

V0=zeros(1,niter);
V1=zeros(1,niter);

SM=zeros(N+2,1);

% LU factorization of matrix A

[L,U]=lu(A);

sav=1;
% pl : used to store |u^n|
pl=zeros(niter/25,N+2); 
pl(1,:)=u1.';
tl=zeros(niter/25,1);
tl(1)=0;
% ERR : relative error in L2 norm
K=[0];ERR=[0];tmp=0;
% l2 norm computed by Simpson quadrature rule
l2=l2simpson(dx,abs(u1));
% Vector to store the evolution of the L2 norm of the approximate
% solution
L2NORM=[l2];
% Vector to store the evolution of the L2 norm of the exact solution
L2NORMEX=[l2];
  
%--------------------------------------------------------------
%--------------------------------------------------------------

% Iterations
for k=1:niter,
% u1 means u^n
% u2 means u^{n+1}
% Resolution of the linear system
  y=L\(B*u1+SM);
  u2=U\y;
  u1=u2;
% Storage of the boundary terms
  V0(k) =u2(1);%W0=[W0 u2(2)];
  V1(k) =u2(N+2);%W1=[W1 u2(N+1)];
% Computation of the futur R.H.S by convolution
  sam=s(2:k+1);
  SM(1)=fliplr(V0(1:k))*sam-u2(2);
  SM(N+2)=fliplr(V1(1:k))*sam-u2(N+1);
  
% Display
  if rem(k,25)==0
% $$$       fprintf('Iteration %d en %20.15f sec\n',k,cputime-tmp);
% $$$       u_exact = sqrt(i/(-4*dt*k+i)).*exp((-i*x.^2-k0*x+k0^2*dt*k)/(-4*dt*k+i));
% $$$       K=[K;k*dt];
% $$$       %ERR=[ERR;max(abs(u1)-abs(u_exact))];
% $$$       l2=l2simpson(dx,abs(u1));
% $$$       l2ex=l2simpson(dx,abs(u_exact));
% $$$       L2NORM=[L2NORM;l2];
% $$$       L2NORMEX=[L2NORMEX;l2ex];
% $$$       u_eccard=u1-u_exact;
% $$$       ERR=[ERR;l2simpson(dx,abs(u_eccard))/l2ex];      
    sav=sav+1;
    pl(sav,:)=u1.';
    tl(sav)=k*dt;
    if   data.PlotFlag
      plot(x,abs(u1),'parent',data.axe1);axis(data.axe1,[x0 x1 0 1]);drawnow;
      set(data.txtiter,'String',num2str(k));
      if rem(sav,20)==0
	pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
	contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
		 'parent',data.axe2);
	axis(data.axe2,[x0 x1 0 tfinal]);
	h=colorbar('peer',data.axe2);
	set(h,'fontsize',7);
      end
% $$$       tmp=cputime;
    end
  end
end

if data.PlotFlag
  pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
  contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
           'parent',data.axe2);
  axis(data.axe2,[x0 x1 0 tfinal]);
  h=colorbar('peer',data.axe2);
  set(h,'fontsize',7);
end

% Output
if data.ReturnFlag == 1
  out = pl;
elseif data.ReturnFlag == 2
  out.pl = pl;
  out.tl = tl;
  out.gridx = gridx;
else
  error(sprintf('Unknown ReturnFlag %g',ReturnFlag))
end