function [out]=funcfd(data)
% Antoine-Besse July 2006
% Computation of the solution of the free Schroedinger equation
% by Crank-Nicolson scheme with a linear finite element method
% with Absorbing Boundary conditions : discretization of Antoine Besse
% with Dirichlet-Neuman formulation.

% data is a cell
%  |
%  |--- data.a
%  |--- data.b
%  |--- data.sigma
%  |--- data.xc
%  |--- data.x0
%  |--- data.x1
%  |--- data.N
%  |--- data.tfinal
%  |--- data.dt
%  |--- data.k0  
%  |--- data.m
%  |--- data.initDatum
%  |--- data.axe1  : where to plot |u|
%  |--- data.axe2  : where to plot contour of log10|u|
%  |--- data.txtiter    : handle on textiter
%  |--- data.txtnbiter  : handle on textnbiter

%  clear all;
if ~isfield(data,'PlotFlag')
  data.PlotFlag = logical(1);
end
if ~isfield(data,'ReturnFlag')
  data.ReturnFlag = 1;
end

% Data
% Computational domain : [x0,x1]
% N+2 points : extremity set on points 1 and N+2
savenb=data.savenb;
a  = data.a;
b  = data.b;
sigma = data.sigma;
xc = data.xc;
x0 = data.x0; 
x1 = data.x1;
N  = data.N;
dx = (x1-x0)/(N+1);
% Final time of computation
% dt : time step
% niter : number of iterations
tfinal = data.tfinal;
dt = data.dt;
niter = tfinal/dt;
if data.PlotFlag
  set(data.txtnbiter,'String',num2str(niter));  
end
% Wave number
k0=data.k0;

x = (x0:dx:x1)';
gridx = x;

% Initial datum
%u=initialDatum(x,k0);
u = eval(data.initDatum); 

% try
%   V = potential_func(data.xc);
%   a = data.a;
%   b = data.b;
%   xc = data.xc;
%   k0 = data.k0;
%   sigma = data.sigma;
%   t = 0;
%   u = eval(data.exactCpot); 
% catch
%    warning('Using initDatum') 
% end

% E=exp(I*Pi/4)
E=(sqrt(2)+i*sqrt(2))/2;


A = dx*speye(N+2,N+2);


B = -1/dx*spdiags([ones(N+2,1),-2*ones(N+2,1),ones(N+2,1)],-1:1,N+2,N+2);
B(1,1) = 2/dx;
B(1,2) = -2/dx;
B(end,end) = 2/dx;
B(end,end-1) = -2/dx;

boundaryTerm = -sqrt(2/dt)/E;
C = sparse(N+2,N+2);
C(1,1) = boundaryTerm;
C(end,end) = boundaryTerm;

D = dx*spdiags(potential_func(x),0,N+2,N+2);

% Assembly : matrices of the system
M1 = a*i*A/dt-b*B/2+b*sqrt(a/b)*C-D/2;
M2 = a*i*A/dt+b*B/2-b*sqrt(a/b)*C+D/2;
%
% LU factorization of M1 matrix
%
%[L,U]=lu(M1);
[LA,UA,pA,qA] = lu(M1,'vector');
%
% Computation of convolution terms
%
alpha=convolTerms(niter+1,E,dt);
%
% Remember the boundary values
Vl = potential_func(x0)/a; % Potential value on xl
Vr = potential_func(x1)/a; % Potential value on xr
Vlt = [u(1)*exp(i*Vl*0) zeros(1,niter)];
Vrt = [u(N+2)*exp(i*Vr*0) zeros(1,niter)];
SM_old = zeros(N+2,1);
SM_new = SM_old;

sav=1;
pl=zeros(niter/savenb,N+2);
pl(1,:)=u.';   % Vector to store the evolution of u
% Useful to plot the log-error
tl=zeros(niter/savenb,1);
tl(1)=0;

MaxV=max(potential_func(x));
if (MaxV==0)
  MaxV=1;
end
% Iteration : Crank Nicolson scheme
for k=1:niter,

  SM=b*sqrt(a/b)*sqrt(2/dt)/E*(SM_new+SM_old);
  
%   y=L\(M2*u+SM);
%   u=U\y;
  rhs = M2*u+SM;
  u(qA,:)= UA\(LA\rhs(pA,:));
  
  Vlt(k+1) = u(1)*exp(i*Vl*k*dt);
  Vrt(k+1) = u(N+2)*exp(i*Vr*k*dt);
  
  al = alpha(2:k+1);
  SM_old = SM_new;
  SM_new(1)   = exp(-i*Vl*(k+1)*dt)*fliplr(Vlt(2:k+1))*al;
  SM_new(N+2) = exp(-i*Vr*(k+1)*dt)*fliplr(Vrt(2:k+1))*al;
  
  if rem(k,savenb)==0
    sav=sav+1;
    pl(sav,:)=u.';
    tl(sav)=k*dt;
    % Drawing
    if   data.PlotFlag
      plot(x,abs(u),x,potential_func(x)/MaxV,'r:','parent',data.axe1);axis(data.axe1,[x0 x1 0 1.1]);drawnow;
      set(data.txtiter,'String',num2str(k));
      if rem(sav,20)==0
	pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
	contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
		 'parent',data.axe2);
	axis(data.axe2,[x0 x1 0 tfinal]);
	h=colorbar('peer',data.axe2);
	set(h,'fontsize',7);
      end
    end    
  end
end

% Drawing
if   data.PlotFlag
  pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
  contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
	   'parent',data.axe2);
  axis(data.axe2,[x0 x1 0 tfinal]);
  h=colorbar('peer',data.axe2);
  set(h,'fontsize',7);
end
% Output
if data.ReturnFlag == 1
  out = pl;
elseif data.ReturnFlag == 2
  out.pl = pl;
  out.tl = tl;
  out.gridx = gridx;
else
  error(sprintf('Unknown ReturnFlag %g',ReturnFlag))
end 