function [out]=funcshibata(data)
% Antoine-Besse July 2006
% Computation of the solution of the free Schroedinger equation
% by Shibata scheme with Absorbing Boundary conditions

% data is a cell
%  |
%  |--- data.a
%  |--- data.b
%  |--- data.sigma
%  |--- data.xc
%  |--- data.x0
%  |--- data.x1
%  |--- data.N
%  |--- data.tfinal
%  |--- data.dt
%  |--- data.k0  
%  |--- data.m
%  |--- data.initDatum
%  |--- data.axe1  : where to plot |u|
%  |--- data.axe2  : where to plot contour of log10|u|
%  |--- data.txtiter    : handle on textiter
%  |--- data.txtnbiter  : handle on textnbiter

%  clear all;
% Setting default values
if ~isfield(data,'PlotFlag')
  data.PlotFlag = logical(1);
end
if ~isfield(data,'ReturnFlag')
  data.ReturnFlag = 1;
end

% Data
% Computational domain : [x0,x1]
% N+2 points : extremity set on points 1 and N+2
a=data.a;
b=data.b;
savenb=data.savenb;
sigma=data.sigma;
xc=data.xc;
x0=data.x0; 
x1=data.x1;
N=data.N;
dx=(x1-x0)/(N+1);
% Final time of computation
% dt : time step
% niter : number of iterations
tfinal=data.tfinal;
dt=data.dt;
niter=tfinal/dt;
if data.PlotFlag
  set(data.txtnbiter,'String',num2str(niter));  
end

% Wave number
k0=data.k0;


x=(x0:dx:x1)';
gridx = x;

% Initial datum
%u1=initialDatum(x,k0);
u1=eval(data.initDatum);  
% try
%   V = potential_func(data.xc);
%   a = data.a;
%   b = data.b;
%   xc = data.xc;
%   k0 = data.k0;
%   sigma = data.sigma;
%   t = 0;
%   u1 = eval(data.exactCpot); 
% catch
%   warning('Using initDatum') 
% end

u2=u1;
Vl=potential_func(x0); % Potential value on xl
Vr=potential_func(x1); % Potential value on xr


%-----------------------------------------------------------
% Matrices assembly
%-----------------------------------------------------------
hbar=a;m=a^2/b/2;
pot=potential_func(x);
e=ones(N+2,1);
e1=b*(0.5/dx^2)*e;
e2=(i*a/dt-b/dx^2)*e;
e3=(i*a/dt+b/dx^2)*e;
A=spdiags([e1 e2-pot/2 e1],[-1 0 1],N+2,N+2);

%-----------------------------------------------------------
%Shibata
a1=hbar*abs(k0)/m;a2=a1;c1=2/(a1+a2);c2=m*a1*a2/(a1+a2);
A(1,1)=-0.5*i*hbar/dx-0.5*hbar*c1*i/dt+(c1*Vl-c2)/4;
A(1,2)=0.5*i*hbar/dx-0.5*hbar*c1*i/dt+(c1*Vl-c2)/4;
A(N+2,N+2)=-0.5*i*hbar/dx-0.5*hbar*c1*i/dt+(c1*Vr-c2)/4;
A(N+2,N+1)=0.5*i*hbar/dx-0.5*hbar*c1*i/dt+(c1*Vr-c2)/4;

%
B=spdiags([-e1 e3+pot/2 -e1],[-1 0 1],N+2,N+2);
%-----------------------------------------------------------
%Shibata
B(1,1)=0.5*i*hbar/dx-0.5*c1*hbar*i/dt-(c1*Vl-c2)/4;
B(1,2)=-0.5*i*hbar/dx-0.5*c1*hbar*i/dt-(c1*Vl-c2)/4;
B(N+2,N+2)=0.5*i*hbar/dx-0.5*c1*hbar*i/dt-(c1*Vr-c2)/4;
B(N+2,N+1)=-0.5*i*hbar/dx-0.5*c1*hbar*i/dt-(c1*Vr-c2)/4;

% LU factorization of matrix A

[L,U]=lu(A);
[LA,UA,pA,qA] = lu(A,'vector');

sav=1;
% pl : used to store |u^n|
pl=zeros(floor(niter/savenb),N+2); 
pl(1,:)=u1.';
tl=zeros(floor(niter/savenb),1);
tl(1)=0;
% ERR : relative error in L2 norm
K=[0];ERR=[0];tmp=0;
% l2 norm computed by Simpson quadrature rule
l2=l2simpson(dx,abs(u1));
% Vector to store the evolution of the L2 norm of the approximate
% solution
L2NORM=[l2];
% Vector to store the evolution of the L2 norm of the exact solution
L2NORMEX=[l2];

%--------------------------------------------------------------
%--------------------------------------------------------------

MaxV=max(potential_func(x));
if (MaxV==0)
  MaxV=1;
end;
% Iterations
for k=1:niter,
% u1 means u^n
% u2 means u^{n+1}
% Resolution of the linear system
  rhs = B*u1;
  u2(qA,:)= UA\(LA\rhs(pA,:));
%   y=L\(B*u1);
%   u2=U\y;
% Replacement of the old u^n by u^(n+1)
  u1=u2;
  
% Display
  if rem(k,savenb)==0
% $$$       fprintf('Iteration %d en %20.15f sec\n',k,cputime-tmp);
% $$$       u_exact = sqrt(i/(-4*dt*k+i)).*exp((-i*x.^2-k0*x+k0^2*dt*k)/(-4*dt*k+i));
% $$$       K=[K;k*dt];
% $$$       %ERR=[ERR;max(abs(u1)-abs(u_exact))];
% $$$       l2=l2simpson(dx,abs(u1));
% $$$       l2ex=l2simpson(dx,abs(u_exact));
% $$$       L2NORM=[L2NORM;l2];
% $$$       L2NORMEX=[L2NORMEX;l2ex];
% $$$       u_eccard=u1-u_exact;
% $$$       ERR=[ERR;l2simpson(dx,abs(u_eccard))/l2ex];      
    sav=sav+1;
    pl(sav,:)=u1.';
    tl(sav)=k*dt;
    if   data.PlotFlag
      plot(x,abs(u1),x,potential_func(x)/MaxV,'r:','parent',data.axe1);axis(data.axe1,[x0 x1 0 1.1]);drawnow;
      set(data.txtiter,'String',num2str(k));
      if rem(sav,20)==0
	pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
	contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
		 'parent',data.axe2);
	axis(data.axe2,[x0 x1 0 tfinal]);
	h=colorbar('peer',data.axe2);
	set(h,'fontsize',7);
      end
% $$$       tmp=cputime;
    end
  end
end

if   data.PlotFlag
  pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
  contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
	   'parent',data.axe2);
  axis(data.axe2,[x0 x1 0 tfinal]);
  h=colorbar('peer',data.axe2);
  set(h,'fontsize',7);
end
% Output
if data.ReturnFlag == 1
  out = pl;
elseif data.ReturnFlag == 2
  out.pl = pl;
  out.tl = tl;
  out.gridx = gridx;
else
  error(sprintf('Unknown ReturnFlag %g',ReturnFlag))
end    